@@ -24,10 +24,16 @@ tools for BN analysis in jupyter notebook
 from __future__ import print_function
 
 import time
+import sys
 
 import IPython.display
 import re
-import base64
+
+# fix DeprecationWarning of base64.encodestring()
+try:
+  from base64 import encodebytes
+except ImportError:  # 3+
+  from base64 import encodestring as encodebytes
 
 import matplotlib as mpl
 import matplotlib.pyplot as plt
@@ -42,11 +48,19 @@ from IPython.display import display, HTML, SVG
 
 import pyAgrum as gum
 from pyAgrum.lib.bn2graph import BN2dot, BNinference2dot
+from pyAgrum.lib.id2graph import ID2dot, LIMIDinference2dot
 from pyAgrum.lib.mn2graph import MN2UGdot, MNinference2UGdot
 from pyAgrum.lib.mn2graph import MN2FactorGraphdot, MNinference2FactorGraphdot
 from pyAgrum.lib.bn_vs_bn import GraphicalBNComparator
 from pyAgrum.lib.proba_histogram import proba2histo
 
+# check if an instance of ipython exists
+try:
+  get_ipython
+except NameError as e:
+  raise ImportError(
+    "[pyAgrum ERROR] pyAgrum.lib.notebook has to be import from an IPython's instance (mainly notebook).") from None
+
 _cdict = {
   'red': ((0.0, 0.1, 0.3),
           (1.0, 0.6, 1.0)),
@@ -258,7 +272,7 @@ def showJunctionTree(bn, withNames=True, size=None):
   """
   Show a junction tree
 
-  :param bn: the bayesian network
+  :param bn: the Bayesian network
   :param boolean withNames: display the variable names or the node id in the clique
   :param size: size of the rendered graph
   :return: the representation of the graph
@@ -274,101 +288,22 @@ def showJunctionTree(bn, withNames=True, size=None):
     return showDot(jt.toDot(), size)
 
 
-def getJunctionTree(bn, withNames=True, size=None):
+def getJunctionTree(bn, size=None):
   """
   get a HTML string for a junction tree (more specifically a join tree)
 
-  :param bn: the bayesian network
-  :param boolean withNames: display the variable names or the node id in the clique
+  :param bn: the Bayesian network
   :param size: size of the rendered graph
   :return: the HTML representation of the graph
   """
   if size is None:
-    size = gum.config["notebook", "default_graph_size"]
+    size = gum.config["notebook", "junctiontree_graph_size"]
 
   jtg = gum.JunctionTreeGenerator()
+  jtg._model = bn
   jt = jtg.junctionTree(bn)
-  if withNames:
-    return getDot(jt.toDotWithNames(bn), size)
-  else:
-    return getDot(jt.toDot(), size)
-
-
-def _infdiag_todot(diag):
-  res = "digraph {"
-
-  # chance node
-  res += '''
-    node [fillcolor="{}",
-          fontcolor="{}",
-          style=filled,shape={}];
-  '''.format(gum.config["influenceDiagram", "default_chance_bgcolor"],
-             gum.config["influenceDiagram", "default_chance_fgcolor"],
-             gum.config["influenceDiagram", "default_chance_shape"])
-  for node in diag.nodes():
-    if diag.isChanceNode(node):
-      res += '   "' + diag.variable(node).name() + '";' + "\n"
-
-  # decision node
-  res += '''
-    node [fillcolor="{}",
-          fontcolor="{}",
-          style=filled,shape={}];
-  '''.format(gum.config["influenceDiagram", "default_decision_bgcolor"],
-             gum.config["influenceDiagram", "default_decision_fgcolor"],
-             gum.config["influenceDiagram", "default_decision_shape"])
-  for node in diag.nodes():
-    if diag.isDecisionNode(node):
-      res += '   "' + diag.variable(node).name() + '";' + "\n"
-
-  # utility node
-  res += '''
-    node [fillcolor="{}",
-          fontcolor="{}",
-          style=filled,shape={}];
-  '''.format(gum.config["influenceDiagram", "default_utility_bgcolor"],
-             gum.config["influenceDiagram", "default_utility_fgcolor"],
-             gum.config["influenceDiagram", "default_utility_shape"])
-  for node in diag.nodes():
-    if diag.isUtilityNode(node):
-      res += '   "' + diag.variable(node).name() + '";' + "\n"
-
-  # arcs
-  res += "\n"
-  for node in diag.nodes():
-    for chi in diag.children(node):
-      res += '  "' + diag.variable(node).name() + '"->"' + \
-             diag.variable(chi).name() + '";' + "\n"
-  res += "}"
-  return res
-
-
-def showInfluenceDiagram(diag, size=None):
-  """
-  show an influence diagram as a graph
-
-  :param diag: the influence diagram
-  :param size: size of the rendered graph
-  :return: the representation of the influence diagram
-  """
-  if size is None:
-    size = gum.config["notebook", "default_graph_size"]
-
-  return showDot(_infdiag_todot(diag), size)
-
-
-def getInfluenceDiagram(diag, size=None):
-  """
-  get a HTML string for an influence diagram as a graph
-
-  :param diag: the influence diagram
-  :param size: size of the rendered graph
-  :return: the HTML representation of the influence diagram
-  """
-  if size is None:
-    size = gum.config["notebook", "default_graph_size"]
-
-  return getDot(_infdiag_todot(diag), size)
+  jt._engine = jtg
+  return getDot(getJT(jt, size), size)
 
 
 def showProba(p, scale=1.0):
@@ -379,7 +314,7 @@ def showProba(p, scale=1.0):
   :return:
   """
   fig = proba2histo(p, scale)
-  fig.patch.set_facecolor(gum.config["notebook", "figure_facecolor"])
+  #  fig.patch.set_facecolor(gum.config["notebook", "figure_facecolor"])
   IPython.display.set_matplotlib_formats(
     gum.config["notebook", "graph_format"])
   plt.show()
@@ -492,11 +427,39 @@ def showMN(mn, view=None, size=None, nodeColor=None, factorColor=None, edgeWidth
   return showGraph(dottxt, size)
 
 
+def showInfluenceDiagram(diag, size=None):
+  """
+  show an influence diagram as a graph
+
+  :param diag: the influence diagram
+  :param size: size of the rendered graph
+  :return: the representation of the influence diagram
+  """
+  if size is None:
+    size = gum.config["influenceDiagram", "default_id_size"]
+
+  return showGraph(ID2dot(diag), size)
+
+
+def getInfluenceDiagram(diag, size=None):
+  """
+  get a HTML string for an influence diagram as a graph
+
+  :param diag: the influence diagram
+  :param size: size of the rendered graph
+  :return: the HTML representation of the influence diagram
+  """
+  if size is None:
+    size = gum.config["influenceDiagram", "default_id_size"]
+
+  return getGraph(ID2dot(diag), size)
+
+
 def showBN(bn, size=None, nodeColor=None, arcWidth=None, arcColor=None, cmap=None, cmapArc=None):
   """
   show a Bayesian network
 
-  :param bn: the bayesian network
+  :param bn: the Bayesian network
   :param size: size of the rendered graph
   :param nodeColor: a nodeMap of values (between 0 and 1) to be shown as color of nodes (with special colors for 0 and 1)
   :param arcWidth: a arcMap of values to be shown as width of arcs
@@ -552,7 +515,7 @@ def getBN(bn, size=None, nodeColor=None, arcWidth=None, arcColor=None, cmap=None
   """
   get a HTML string for a Bayesian network
 
-  :param bn: the bayesian network
+  :param bn: the Bayesian network
   :param size: size of the rendered graph
   :param nodeColor: a nodeMap of values (between 0 and 1) to be shown as color of nodes (with special colors for 0 and 1)
   :param arcWidth: a arcMap of values to be shown as width of arcs
@@ -629,7 +592,7 @@ def _reprInformation(bn, evs, size, cmap, asString):
                                   orientation='horizontal')
   cb1.set_label('Entropy')
   png = print_figure(canvas.figure, "png")  # from IPython.core.pylabtools
-  png_legend = "<img style='vertical-align:middle' src='data:image/png;base64,%s'>" % base64.encodestring(png).decode(
+  png_legend = "<img style='vertical-align:middle' src='data:image/png;base64,%s'>" % encodebytes(png).decode(
     'ascii')
 
   gsvg = SVG(gr.create_svg())
@@ -701,7 +664,8 @@ def _get_showInference(model, engine=None, evs=None, targets=None, size=None,
   if isinstance(model, gum.BayesNet):
     if engine is None:
       engine = gum.LazyPropagation(model)
-    return BNinference2dot(model, size, engine, evs, targets, nodeColor=nodeColor, arcWidth=arcWidth, arcColor=arcColor, cmapNode=cmap, cmapArc=cmapArc)
+    return BNinference2dot(model, size, engine, evs, targets, nodeColor=nodeColor, arcWidth=arcWidth, arcColor=arcColor,
+                           cmapNode=cmap, cmapArc=cmapArc)
   elif isinstance(model, gum.MarkovNet):
     if view is None:
       view = gum.config["notebook", "default_markovnetwork_view"]
@@ -709,12 +673,17 @@ def _get_showInference(model, engine=None, evs=None, targets=None, size=None,
         engine = gum.ShaferShenoyMNInference(model)
 
       if view == "graph":
-        return MNinference2UGdot(model, size, engine, evs, targets, nodeColor, factorColor, arcWidth, arcColor, cmap,
-                                 cmapArc)
+        return MNinference2UGdot(model, size, engine, evs, targets, nodeColor=nodeColor, factorColor=factorColor,
+                                 arcWidth=arcWidth, arcColor=arcColor, cmapNode=cmap, cmapArc=cmapArc)
       else:
-        return MNinference2FactorGraphdot(model, size, engine, evs, targets, nodeColor, factorColor, cmap)
-    else:
-      raise gum.InvalidArgument("Argument model should be a PGM (BayesNet or MarkovNet")
+        return MNinference2FactorGraphdot(model, size, engine, evs, targets, nodeColor=nodeColor,
+                                          factorColor=factorColor, cmapNode=cmap)
+  elif isinstance(model, gum.InfluenceDiagram):
+    if engine is None:
+      engine = gum.ShaferShenoyLIMIDInference(model)
+    return LIMIDinference2dot(model, size, engine, evs, targets)
+  else:
+    raise gum.InvalidArgument("Argument model should be a PGM (BayesNet, MarkovNet or Influence Diagram")
 
 
 def showInference(model, engine=None, evs=None, targets=None, size=None,
@@ -724,24 +693,26 @@ def showInference(model, engine=None, evs=None, targets=None, size=None,
   """
   show pydot graph for an inference in a notebook
 
-  :param gum.{Bayes|Markov}Net model: the model in which to infer
-  :param gum.Inference engine: inference algorithm used. If None, LazyPropagation will be used for BayesNet and ShaferShenoy for Markov net
+  :param GraphicalModel model: the model in which to infer (pyAgrum.BayesNet, pyAgrum.MarkovNet or
+          pyAgrum.InfluenceDiagram)
+  :param gum.Inference engine: inference algorithm used. If None, gum.LazyPropagation will be used for BayesNet,
+          gum.ShaferShenoy for gum.MarkovNet and gum.ShaferShenoyLIMIDInference for gum.InfluenceDiagram.
   :param dictionnary evs: map of evidence
   :param set targets: set of targets
   :param string size: size of the rendered graph
   :param nodeColor: a nodeMap of values (between 0 and 1) to be shown as color of nodes (with special colors for 0 and 1)
+  :param factorColor: a nodeMap of values (between 0 and 1) to be shown as color of factors (in MarkovNet representation)
   :param arcWidth: a arcMap of values to be shown as width of arcs
   :param arcColor: a arcMap of values (between 0 and 1) to be shown as color of arcs
   :param cmap: color map to show the color of nodes and arcs
   :param cmapArc: color map to show the vals of Arcs.
-  :param graph : only shows nodes that have their id in the graph (and not in the whole BN)
-  :param view: 'graph' | 'factorgraph’ | None (default) for Markov network
-
+  :param graph: only shows nodes that have their id in the graph (and not in the whole BN)
+  :param view: graph | factorgraph | None (default) for Markov network
   :return: the desired representation of the inference
   """
   grinf = _get_showInference(model, engine, evs, targets, size, nodeColor, arcWidth, arcColor, cmap, cmapArc, graph,
                              view)
-  return showGraph(grinf, size)
+  showGraph(grinf, size)
 
 
 def getInference(model, engine=None, evs=None, targets=None, size=None,
@@ -751,23 +722,26 @@ def getInference(model, engine=None, evs=None, targets=None, size=None,
   """
   get a HTML string for an inference in a notebook
 
-  :param gum.{Bayes|Markov}Net model: the model in which to infer
-  :param gum.Inference engine: inference algorithm used. If None, LazyPropagation will be used for BayesNet and ShaferShenoy for Markov net
+  :param GraphicalModel model: the model in which to infer (pyAgrum.BayesNet, pyAgrum.MarkovNet or
+          pyAgrum.InfluenceDiagram)
+  :param gum.Inference engine: inference algorithm used. If None, gum.LazyPropagation will be used for BayesNet,
+          gum.ShaferShenoy for gum.MarkovNet and gum.ShaferShenoyLIMIDInference for gum.InfluenceDiagram.
   :param dictionnary evs: map of evidence
   :param set targets: set of targets
   :param string size: size of the rendered graph
   :param nodeColor: a nodeMap of values (between 0 and 1) to be shown as color of nodes (with special colors for 0 and 1)
+  :param factorColor: a nodeMap of values (between 0 and 1) to be shown as color of factors (in MarkovNet representation)
   :param arcWidth: a arcMap of values to be shown as width of arcs
   :param arcColor: a arcMap of values (between 0 and 1) to be shown as color of arcs
   :param cmap: color map to show the color of nodes and arcs
   :param cmapArc: color map to show the vals of Arcs.
-  :param graph : only shows nodes that have their id in the graph (and not in the whole BN)
-  :param view: 'graph' | 'factorgraph’ | None (default) for Markov network
+  :param graph: only shows nodes that have their id in the graph (and not in the whole BN)
+  :param view: graph | factorgraph | None (default) for Markov network
 
   :return: the desired representation of the inference
   """
-  grinf = _get_showInference(model, engine, evs, targets, size, nodeColor=nodeColor, arcWidth=arcWidth, arcColor=arcColor, cmap=cmap, cmapArc=cmapArc, graph=graph,
-                             view=view)
+  grinf = _get_showInference(model, engine, evs, targets, size, nodeColor=nodeColor, arcWidth=arcWidth,
+                             arcColor=arcColor, cmap=cmap, cmapArc=cmapArc, graph=graph, view=view)
   return getGraph(grinf, size)
 
 
@@ -776,12 +750,12 @@ def _reprPotential(pot, digits=None, withColors=True, varnames=None, asString=Fa
   return a representation of a gum.Potential as a HTML table.
   The first dimension is special (horizontal) due to the representation of conditional probability table
 
-
   :param pot: the potential to get
   :param digits: number of digits to show
-  :param: withColors : bgcolor for proba cells or not
+  :param withColors: bgcolor for proba cells or not
   :param varnames: the aliases for variables name in the table
   :param asString: display the table or a HTML string
+
   :return: the representation
   """
   from IPython.core.display import HTML
@@ -1059,10 +1033,77 @@ def getInferenceEngine(ie, inferenceCaption):
   return getSideBySide(getBN(ie.BN()), t, captions=[inferenceCaption, "Evidence and targets"])
 
 
+def getJT(jt, size=None):
+  if gum.config["notebook", "junctiontree_with_names"] == "True":
+    cliqlabels = lambda c: " ".join(sorted([model.variable(n).name() for n in jt.clique(c)]))
+    cliqnames = lambda c: "-".join(sorted([model.variable(n).name() for n in jt.clique(c)]))
+    seplabels = lambda c1, c2: " ".join(sorted([model.variable(n).name() for n in jt.separator(c1, c2)]))
+    sepnames = lambda c1, c2: cliqnames(c1) + '+' + cliqnames(c2)
+  else:
+    cliqlabels = lambda c: " ".join([str(n) for n in sorted(jt.clique(c))])
+    cliqnames = lambda c: "-".join([str(n) for n in sorted(jt.clique(c))])
+    seplabels = lambda c1, c2: " ".join([str(n) for n in sorted(jt.separator(c1, c2))])
+    sepnames = lambda c1, c2: cliqnames(c1) + '^' + cliqnames(c2)
+
+  model = jt._engine._model
+  name = model.propertyWithDefault("name", str(type(model)).split(".")[-1][:-2])
+  graph = dot.Dot(graph_type='graph', graph_name=name, bgcolor="transparent")
+  for c in jt.nodes():
+    graph.add_node(dot.Node('"' + cliqnames(c) + '"',
+                            label='"' + cliqlabels(c) + '"',
+                            style="filled",
+                            fillcolor=gum.config["notebook", "junctiontree_clique_bgcolor"],
+                            fontcolor=gum.config["notebook", "junctiontree_clique_fgcolor"],
+                            fontsize=gum.config["notebook", "junctiontree_clique_fontsize"]))
+  for c1, c2 in jt.edges():
+    graph.add_node(dot.Node('"' + sepnames(c1, c2) + '"',
+                            label='"' + seplabels(c1, c2) + '"',
+                            style="filled",
+                            shape="box", width="0", height="0", margin="0.02",
+                            fillcolor=gum.config["notebook", "junctiontree_separator_bgcolor"],
+                            fontcolor=gum.config["notebook", "junctiontree_separator_fgcolor"],
+                            fontsize=gum.config["notebook", "junctiontree_separator_fontsize"]))
+  for c1, c2 in jt.edges():
+    graph.add_edge(dot.Edge('"' + cliqnames(c1) + '"', '"' + sepnames(c1, c2) + '"'))
+    graph.add_edge(dot.Edge('"' + sepnames(c1, c2) + '"', '"' + cliqnames(c2) + '"'))
+
+  if size is None:
+    size = gum.config["notebook", "default_graph_size"]
+  graph.set_size(gum.config["notebook", "junctiontree_graph_size"])
+
+  return graph.to_string()
+
+
+def getCliqueGraph(cg, size=None):
+  """get a representation for clique graph. Special case for junction tree
+  (clique graph with an attribute `_engine`)
+
+  Args:
+      cg (gum.CliqueGraph): the clique graph (maybe junction tree for a _model) to
+                            represent
+  """
+  if hasattr(cg, "_engine"):
+    return getDot(getJT(cg), size)
+  else:
+    return getDot(cg.toDot())
+
+
+# hook to control some parameters for notebook
+def _update_config():
+  mpl.rcParams['figure.facecolor'] = gum.config["notebook", "figure_facecolor"]
+  IPython.display.set_matplotlib_formats(gum.config["notebook", "graph_format"])
+
+
+gum.config.add_hook(_update_config)
+gum.config.run_hooks()
+
 # adding _repr_html_ to some pyAgrum classes !
 gum.BayesNet._repr_html_ = lambda self: getBN(self)
 gum.MarkovNet._repr_html_ = lambda self: getMN(self)
 gum.BayesNetFragment._repr_html_ = lambda self: getBN(self)
+gum.InfluenceDiagram._repr_html_ = lambda self: getInfluenceDiagram(self)
+
+gum.CliqueGraph._repr_html_ = lambda self: getCliqueGraph(self)
 
 gum.Potential._repr_html_ = lambda self: getPotential(self)
 gum.LazyPropagation._repr_html_ = lambda self: getInferenceEngine(
@@ -1072,10 +1113,5 @@ gum.UndiGraph._repr_html_ = lambda self: getDot(self.toDot())
 gum.DiGraph._repr_html_ = lambda self: getDot(self.toDot())
 gum.MixedGraph._repr_html_ = lambda self: getDot(self.toDot())
 gum.DAG._repr_html_ = lambda self: getDot(self.toDot())
-gum.CliqueGraph._repr_html_ = lambda self: getDot(self.toDot())
 gum.EssentialGraph._repr_html_ = lambda self: getDot(self.toDot())
 gum.MarkovBlanket._repr_html_ = lambda self: getDot(self.toDot())
-gum.InfluenceDiagram._repr_html_ = lambda self: getDot(_infdiag_todot(self))
-
-mpl.rcParams['figure.facecolor'] = gum.config["notebook", "figure_facecolor"]
-IPython.display.set_matplotlib_formats(gum.config["notebook", "graph_format"])
